/*
 * @(#)Client.java	1.0 2001/03/12
 *
 * Copyright (C) 2001 by Maru Newby
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To contact the author, email ekko@users.sourceforge.net.
 */


package net.sourceforge.journeyftp.adapter;


// Java imports
import java.io.*;
import java.util.*;
import java.text.SimpleDateFormat;
import java.text.DateFormat;
import java.util.Date;

/**
 * Client.java
 *
 * Provides a textual user interface for the ftp client.
 *
 * @author  Maru Newby
 * @version 1.0, 2001/02/01
 */
public class Client implements FileTransferObserver
{
    // Default prompt
    public final static String FTP_PROMPT = "journeyftp> ";

    // Instance variables
    private String      _hostname = null;
    private TransferAdapter   _client   = null;
    private File file = null;
    private FileInputStream fis = null;
    private FileOutputStream fos = null;


    public static void main( String[] args )
    {
        if ( args.length < 3 )
        {
            System.out.println( "Usage Example: <class> <host> <user> <password>." );
            System.exit(0);
        }

        Client ui = null;
        try
        {
            ui = new Client( args[0], args[1], args[2] );
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
            System.out.println( "Unable to initialize the transfer adapter." );
            System.out.println( "Exiting..." );
            System.exit( 0 );
        }
        ui.init();
    }


    /**
     * Constructor taking the hostname to connect to.
     */
    public Client( String hostname, String user, String password ) throws Exception
    {
        _hostname = hostname;
        TransferAdapterInfo[] aInfo = TransferAdapterFactory.getInstance().getAdapterList();
//        for (int i = 0; i < aInfo.length; ++i)
//            System.out.println( aInfo[i].getDescription() );
        _client = TransferAdapterFactory.getInstance().createTransferAdapter(aInfo[0]);
        ((net.sourceforge.journeyftp.adapter.jta.TelnetAdapter)_client).addTransferObserver(this);
        try
        {
            ConnectionInfo info = new ConnectionInfo( hostname, user, password );
            _client.connect(info);
        }
        catch ( Exception ex )
        {
            System.out.println( "Invalid host or credentials" );
            System.exit(0);
        }
    }


    /**
     * Attempt to execute commands entered by the user.
     *
     * @param command       the command to execute
     */
    public void executeCommand( String command ) throws Exception
    {
        command.toLowerCase();
        command = command.trim();
        boolean failed = false;

        if ( command.equals( "ls" ) )
        {
            VirtualFile[] files = null;
            try
            {
                files = _client.list();
                files = _client.list();
                files = _client.list();
            }
            catch ( Exception ex)
            {
                System.out.print( ex.getMessage() );
                failed = true;
            }

            if (!failed)
            {
                if ( files == null || files.length == 0 )
                {
                    System.out.println( "There are no files in the current directory." );
                }
                else
                {
                    DateFormat simpleFormat = new SimpleDateFormat("yyyy/MM/dd  hh:mm");

                    String size;

                    System.out.println( "===============================================================================" );

                    for (int i = 0; i < files.length; i++)
                    {
                        if (files[i].isDirectory())
                            size = "<DIR>         ";
                        else
                            size = Long.toString(files[i].getSize());

                        System.out.println( simpleFormat.format(files[i].getDateModified()) + " " +
                                            rightJustify(size, 21) +
                                            " " + files[i].getName() );
                    }
                    System.out.println( "===============================================================================" );
                    System.out.println( " Total Files: " + files.length );


                }
            }

        }
        else if ( command.startsWith( "cd" ) )
        {
            StringTokenizer tokenizer = new StringTokenizer( command );
            if ( tokenizer.countTokens() < 2 )
            {
                System.out.println( "Please provide a new working directory." );
                return;
            }
            try
            {
                _client.setWorkingDirectory( command.substring( 3 ) );
                _client.setWorkingDirectory( command.substring( 3 ) );
                _client.setWorkingDirectory( command.substring( 3 ) );
            }
            catch ( Exception ex )
            {
                System.out.print( ex.getMessage() );
                failed = true;
            }

            if (!failed)
                System.out.println( "Directory changed to \"" +
                                    _client.getWorkingDirectory() + "\"" );
        }
        else if ( command.equals( "pwd" ) )
        {
            String junk = _client.getWorkingDirectory();
            junk = _client
.getWorkingDirectory();
            System.out.println( "\"" + _client.getWorkingDirectory() + "\"" +
                                " is the current working directory.");
        }
        else if ( command.equals( "quit" ) )
        {
            _client.disconnect();
            System.exit(0);
        }
        else if ( command.startsWith( "mkdir" ) )
        {
            StringTokenizer tokenizer = new StringTokenizer( command );
            if ( tokenizer.countTokens() < 2 )
            {
                System.out.println( "Please provide a directory name." );
                return;
            }
            tokenizer.nextToken();
            String dirName = tokenizer.nextToken();
            try
            {
                _client.createDirectory(dirName);
            }
            catch ( TransferAdapterException tae )
            {
                System.out.print( tae.getMessage() );
                failed = true;
            }
            if (!failed)
                System.out.println( "New directory created: " + dirName );


        }

        else if ( command.startsWith( "put" ) )
        {
            StringTokenizer tokenizer = new StringTokenizer( command );
            if ( tokenizer.countTokens() < 2 )
            {
                System.out.println( "Please provide a filename." );
                return;
            }
            tokenizer.nextToken();
            String filename = tokenizer.nextToken();

            file = new File( filename );

            if ( file.isDirectory() )
            {
                System.out.println( filename + ": Cannot transfer a directory." );
                return;
            }
            else if ( ! file.canRead() )
            {

                System.out.println( filename + ": Unable to read file." );
                return;
            }

            try
            {
                fis = new FileInputStream( file );
                file = null;
                _client.putFile( filename, fis, 0 );
            }
            catch ( FileNotFoundException fnfe )
            {
                System.out.println( filename + ": Unable to read file." );
            }
        }
        else if ( command.startsWith( "get" ) )
        {
            StringTokenizer tokenizer = new StringTokenizer( command );
            if ( tokenizer.countTokens() < 2 )
            {
                System.out.println( "Please provide a filename." );
                return;
            }
            tokenizer.nextToken();
            String filename = tokenizer.nextToken();

            file = new File( filename );

            if ( file.exists() )
            {
                System.out.println( filename + ": Already exists." );
                return;
            }

            FileOutputStream fos = null;

            try
            {
                file.createNewFile();
                fos = new FileOutputStream( file );
                _client.getFile( filename, fos, 0 );
            }
            catch ( FileNotFoundException fnfe )
            {
                System.out.println( filename + ": File not readable." );
            }
            catch (Exception e)
            {
                System.out.println( e.getMessage() );
            }
        }

        else
        {
            System.out.println( "Unknown Command" );
        }

    } // end: executeCommand


    /**
     * Initialize the client by attempting to connect to the server and allow the
     * user to enter commands.
     */
    public void init()
    {
        try
        {
//            System.out.println( _client.open( _hostname, FTPInfo.CTRL_PORT ) );
        }
        catch ( Exception ex )
        {
            System.out.println( "Unable to connect to server " +
                                _hostname + "." );
            System.exit( 0 );
        }

        while(true)
        {
            try
            {
                executeCommand( readCommand() );
            }
            catch ( Exception ex )
            {
                ex.printStackTrace();
                System.out.print( ex.getMessage() );
                System.out.println( "Connection failure." );
                System.out.println( "Terminating..." );
                System.exit( 0 );
            }
        }

    } // end: init


    /**
     * Utility method right justifies a string for display purposes with spaces.
     *
     * @param value         the value to right justify
     * @param length        the length of to pad to
     */
    public String rightJustify(String value, int length)
    {
        int emptyLen = length - value.length();

        char[] buffer = new char[emptyLen];

        for (int i = 0; i < (emptyLen); ++i)
        {
            buffer[i] = ' ';
        }

        return (new StringBuffer().append( buffer ).append( value ).toString() );

    } // end: rightJustify

    /**
     * Reads a command from the user.
     *
     * @return              the command from the user
     *
     */
    public String readCommand()
    {
        BufferedReader console = new BufferedReader(
                                           new InputStreamReader( System.in ) );
        System.out.print( FTP_PROMPT );
        String command = null;
        try
        {
            command = console.readLine();
        }
        catch (IOException e) { command = "<" + e + ">"; }  // This should never happen

        return ( command );

    } // end: readCommand


    /**
     * This method is called every time the <code>FileTransfer</code> object
     * received a block of bytes, allowing observers to monitor the status of
     * a file transfer.
     *
     * @param bytes         the number of bytes received
     */
    public void dataReceived(int bytes)
    {
        System.out.print("*");

    } // end: dataReceived


    /**
     * This method is called every time the <code>FileTransfer</code> object
     * sends a block of bytes, allowing observers to monitor the status of
     * a file transfer.
     *
     * @param bytes         the number of bytes transferred
     */
    public void dataSent(int bytes)
    {
        System.out.print(">");

    } // end: dataSent


    /**
     * Indicates that a file transfer has been completed successfully.
     */
    public void transferSuccessful()
    {
        System.out.println( "\nFile successfully transferred" );
        try
        {
            if (fis != null)
            {
                fis.close();
                fis = null;
            }
            if (fos != null)
            {
                fos.close();
                fos = null;
            }
        }
        catch (IOException ioe)
        {
            
        }

    } // end: transferSuccessful


    /**
     * Indicates that a file transfer has terminated.  The exception can be
     * used to determine why the transfer terminated.
     *
     * @param ex        the exception indicating the reason for the termination
     */
    public void transferTerminated(TransferAdapterException tae)
    {
        System.out.print( tae.getMessage() );
        if (file != null)
        {
            if (!file.delete())
            file = null;
        }

    } // end: transferTerminated


} // end: Client

