/*
 * @(#)ConnectionInfo.java 1.0 2000/03/05
 *
 * Copyright (C) 2001 by Maru Newby
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To contact the author, email ekko@users.sourceforge.net.
 */


package net.sourceforge.journeyftp.adapter;


/**
 * This class encapsulates the information necessary for a
 * <code>TransferAdapter</code> to make a connection with a server.
 *
 * @author  Maru Newby
 * @version 1.0, 2001/03/12
 */
public class ConnectionInfo implements java.io.Serializable
{
    // Instance variables
    protected String _host;
    protected String _username;
    protected String _password;
    protected String _protocol;
    protected String _workgroup;
    protected String _shareName;
    protected boolean _connected;
    /**
     * Default constuctor - initializes all values to empty strings.
     */
    public ConnectionInfo()
    {
        this("", "", "", "", "", "",false);
    }


    /**
     * Constucts a <code>ConnectionInfo</code> object and initializes its values
     * to those provided.
     *
     * @param host          the host to connect to
     * @param username      the username to authenticate with
     * @param password      the password to authenticate with
     */
    public ConnectionInfo(String host, String username, String password)
    {
        setHost(host);
        setUsername(username);
        setPassword(password);
    }

        public ConnectionInfo(String host, String username, String password, String protocol)
    {
        setHost(host);
        setUsername(username);
        setPassword(password);
        setProtocol(protocol);
    }

        public ConnectionInfo(String host, String username, String password, String protocol, String workgroup, String shareName)
    {
        setHost(host);
        setUsername(username);
        setPassword(password);
        setProtocol(protocol);
        setWorkgroup(workgroup);
        setShareName(shareName);

    }

            public ConnectionInfo(String host, String username, String password, String protocol, String workgroup, String shareName, boolean connected)
    {
        setHost(host);
        setUsername(username);
        setPassword(password);
        setProtocol(protocol);
        setWorkgroup(workgroup);
        setShareName(shareName);
        setConnected(connected);

    }

    public boolean getConnected()
    {
    	return _connected;
    }


    public String getProtocol()
    {
        return (_protocol);
    }

    public String getWorkgroup()
    {
        return (_workgroup);

    }

    public String getShareName()
    {
        return (_shareName);

    }


    /**
     * Accessor for the host.
     *
     * @return              the host to connect to
     */
    public String getHost()
    {
        return (_host);

    } // end: getHost


    /**
     * Accessor for the password.
     *
     * @return              the password to authenticate with
     */
    public String getPassword()
    {
        return (_password);

    } // end: getPassword


    /**
     * Accessor for the username.
     *
     * @param username      the username to authenticate with
     */
    public String getUsername()
    {
        return (_username);

    } // end: getUsername


    public void setConnected(boolean connected)
    {
    	_connected = connected;
    }

    public void setProtocol(String protocol)
    {
        _protocol = protocol;

    }

    public void setWorkgroup(String workgroup)
    {
        _workgroup = workgroup;

    }

    public void setShareName(String shareName)
    {
        _shareName = shareName;

    }


    /**
     * Mutator for the host.
     *
     * @param host          the host to connect to
     */
    public void setHost(String host)
    {
        _host = host;

    } // end: setHost


    /**
     * Mutator for the password.
     *
     * @param password      the password to authenticate with
     */
    public void setPassword(String password)
    {
        _password = password;

    } // end: setPassword


    /**
     * Mutator for the username.
     *
     * @return              the username to authenticate with
     */
    public void setUsername(String username)
    {
        _username = username;

    } // end: setUsername


} // end: ConnectionInfo
