/*
 * @(#)FileTransferObserver.java	1.0 2001/03/12
 *
 * Copyright (C) 2001 by Maru Newby
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To contact the author, email ekko@users.sourceforge.net.
 */


package net.sourceforge.journeyftp.adapter;


/**
 * This class is the observer interface for a file transfer.  It enables the
 * observer to be notified when bytes are transferred, and when a transfer
 * session is completed successfully or terminated.
 *
 * @author  Maru Newby
 * @version 1.0, 2001/03/12
 */
public interface FileTransferObserver
{


    /**
     * This method is called every time the <code>FileTransfer</code> object
     * transfers a block of bytes, allowing observers to monitor the status of
     * a file transfer.
     *
     * @param bytes         the number of bytes transferred
     */
    public void dataTransferred(long bytes);


    /**
     * Indicates that a file transfer has been completed successfully.
     */
    public void transferSuccessful();


    /**
     * Indicates that a file transfer has terminated.  The exception can be
     * used to determine why the transfer terminated.
     *
     * @param ex        the exception indicating the reason for the termination
     */
    public void transferTerminated(TransferAdapterException ex);


} // end: FileTransferObserver
