/*
 * @(#)FileTransferSubject.java	1.0 2001/03/12
 *
 * Copyright (C) 2001 by Maru Newby
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To contact the author, email ekko@users.sourceforge.net.
 */


package net.sourceforge.journeyftp.adapter;


/**
 * Defines an abstract subject interface to complement the
 * <code>FileTransferObserver</code> interface.
 *
 * @author  Maru Newby
 * @version 1.0, 2001/03/12
 */
public interface FileTransferSubject
{


    /**
     * Adds an observer.
     *
     * @param observer      the observer to add
     */
    public void addObserver(FileTransferObserver observer);


    /**
     * Notifies observers when bytes are received.
     *
     * @param bytes         the number of bytes transferred
     */
    public void notifyDataReceived(int bytes);


    /**
     * Notifies observers when bytes are sent.
     *
     * @param bytes         the number of bytes transferred
     */
    public void notifyDataSent(int bytes);


    /**
     * Notifies observers when the transfer is completed successfully.
     */
    public void notifyTransferSuccessful();


    /**
     * Notifies observers when the transfer is terminated.
     *
     * @param ex            the exception that terminated the transfer
     */
    public void notifyTransferTerminated(TransferAdapterException ex);


    /**
     * Removes an observer.
     *
     * @param observer      the observer to remove
     */
    public void removeObserver(FileTransferObserver observer);


} // end: FileTransferSubject
