/*
 * @(#)TransferAdapter.java	1.0 2000/03/05
 *
 * Copyright (C) 2001 by Maru Newby
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To contact the author, email ekko@users.sourceforge.net.
 */


package net.sourceforge.journeyftp.adapter;


/**
 * This class encapsulates an adapters class and a descriptive string that
 * may be useful for displaying adapter choices to a user.  It is easy
 * to display in a list since its toString method is overridden to return
 * the descriptive string.
 *
 * @author  Maru Newby
 * @version 1.0, 2001/03/12
 */
public class TransferAdapterInfo
{
    // Instance variables
    protected String _className;
    protected String _description;
    protected int    _priority;


    /**
     * Creates an info object with default values.
     */
    public TransferAdapterInfo()
    {
        this("", "", 0);
    }


    /**
     * Creates an info object and initializes its class name, description, and
     * priority
     *
     * @param className     the class name of the adapter
     * @param description   the description of the adapter
     * @param priority      the priority of the adapter
     */
    public TransferAdapterInfo(String className,
                               String description,
                               int priority)
    {
        setClassName(className);
        setDescription(description);
        setPriority(priority);
    }


    /**
     * Accessor for the class name.
     *
     * @return              the class name of the adapter
     */
    public String getClassName()
    {
        return (_className);

    } // end: getClassName


    /**
     * Accessor for the description.
     *
     * @return              the class name of the adapter
     */
    public String getDescription()
    {
        return (_description);

    } // end: getDescription


    /**
     * Accessor for the adapter priority.
     *
     * @return              the adapter priority
     */
    public int getPriority()
    {
        return (_priority);

    } // end: getPriority


    /**
     * Mutator for the class name.
     *
     * @param className     the class name of the adapter
     */
    public void setClassName(String className)
    {
        _className = className;

    } // end: setClassName


    /**
     * Mutator for the description.
     *
     * @param description   the class name of the adapter
     */
    public void setDescription(String description)
    {
        _description = description;

    } // end: setDescription


    /**
     * Mutator for the adapter priority.
     *
     * @param priority      the adapter priority
     */
    public void setPriority(int priority)
    {
        _priority = priority;

    } // end: setPriority


    /**
     * Overridden toString method returns the description of the adapter.
     *
     * @return              the description of the adapter
     */
    public String toString()
    {
        return (_description);

    } // end: toString

} // end: TransferAdapterInfo