/*
 * @(#)VirtualFile.java	1.0 2001/03/12
 *
 * Copyright (C) 2001 by Maru Newby
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To contact the author, email ekko@users.sourceforge.net.
 */


package net.sourceforge.journeyftp.adapter;


// Java imports
import java.util.Date;


/**
 * Encapsulates the properties of a remote file listed by a
 * <code>TransferAdapter</code>.
 *
 * @author  Maru Newby
 * @version 1.0, 2001/03/12
 */
public class VirtualFile
{
    // Instance variables.
    private Date    _dateModified;
    private boolean _isDirectory;
    private String  _name;
    private long    _size;
    private String _residentDir;
    private String _destinationDir;
	private String _abFilename;
    /**
     * Constructs a <code>VirtualFile</code> object with blank initial values.
     */
    public VirtualFile()
    {
        this("", 0, new Date(), false, "", "");
    }

    public VirtualFile(VirtualFile oldfile)
    {
    	setName(oldfile.getName());
    	setSize(oldfile.getSize());
    	setIsDirectory(oldfile.isDirectory());
    	setDateModified(oldfile.getDateModified());
	setResidentDir(oldfile.getResidentDir());
	setDestinationDir(oldfile.getDestinationDir());
    }

    /**
     * Constructs a <code>VirtualFile</code> object with the provided
     * properties.
     *
     * @param name          the name of the file
     * @param size          the size in bytes of the file
     * @param dateModified  the date the file was last modified
     * @param isDirectory   whether the file is a directory     
     */
    public VirtualFile(String name, long size, Date dateModified,
                       boolean isDirectory)
    {
        setName(name);
        setSize(size);
        setDateModified(dateModified);
        setIsDirectory(isDirectory);
    }

        public VirtualFile(String name, long size, Date dateModified,
                       boolean isDirectory, String resDir)
    {
        setName(name);
        setSize(size);
        setDateModified(dateModified);
        setIsDirectory(isDirectory);
        setResidentDir(resDir);
    }

            public VirtualFile(String name, long size, Date dateModified,
                       boolean isDirectory, String resDir, String desDir)
    {
        setName(name);
        setSize(size);
        setDateModified(dateModified);
        setIsDirectory(isDirectory);
        setResidentDir(resDir);
        setDestinationDir(desDir);
    }


    public String getAbsoluteName()
    {
    	return _abFilename;
    }

    public String getDestinationDir()
    {
    	return _destinationDir;
    }

    public String getResidentDir()
    {
    	return _residentDir;
    }


    /**
     * Accessor for the date the file was last modified.
     *
     * @return              the date the file was last modified
     */

    public Date getDateModified()
    {
        return (_dateModified);

    } // end: getDateModified


    /**
     * Accessor for the name of the file.
     *
     * @return              the name of the file
     */
    public String getName()
    {
        return (_name);

    } // end: getName


    /**
     * Accessor for the size of the file.
     *
     * @return              the size of the file
     */
    public long getSize()
    {
        return (_size);

    } // end: getSize


    /**
     * Accessor for whether or not the file is a directory.
     *
     * @return              whether the file is a directory
     */
    public boolean isDirectory()
    {
        return (_isDirectory);

    } // end: isDirectory

    public void setAbsoluteName(String abName)
    {
    	_abFilename = abName;
    }

    public void setDestinationDir(String newDest)
    {
	_destinationDir = newDest;	

    }

    public void setResidentDir(String resDir)
    {
    	_residentDir= resDir;
    }

    /**
     * Mutator for the date the file was last modified.
     *
     * @param dateModified  the date the file was last modified
     */
    public void setDateModified(Date dateModified)
    {
        _dateModified = dateModified;

    } // end: setDateModified


    /**
     * Mutator for whether the file is a directory.
     *
     * @param isDirectory   whether the file is a directory
     */
    public void setIsDirectory(boolean isDirectory)
    {
        _isDirectory = isDirectory;

    } // end: setIsDirectory


    /**
     * Mutator for the name of the file.
     *
     * @return              the name of the file
     */
    public void setName(String name)
    {
        _name = name;

    } // end: setName


    /**
     * Mutator for the size of the file.
     *
     * @return              the size of the file
     */
    public void setSize(long size)
    {
        _size = size;

    } // end: setSize


} // end: VirtualFile
