/*
 * @(#)TelnetResponse.java	1.0 2001/03/21
 *
 * Copyright (C) 2001 by Maru Newby
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To contact the author, email ekko@users.sourceforge.net.
 */


package net.sourceforge.journeyftp.adapter.jta;


/**
 * Provides convenience parsing methods for responses received from the server.
 *
 * @author  Maru Newby
 * @version 1.0, 2001/03/21
 */
public class TelnetResponse
{
    // Instance variables
    private String _response = "";


    /**
     * Constructor taking a string response from a server
     *
     * @param response      the string returned from the server
     */
    public TelnetResponse(String response)
    {
        parse(response);
    }


    /**
     * Retreives the error line (if any) from the given telnet response.
     *
     * @param response      the response from the server
     * @return              the error line from the response
     */
    public String getErrorLine()
    {
        String error;

        if (_response == null)
            error = "";
        else
        {
            int endIndex = _response.indexOf("\n");

            if (endIndex == -1)
                error = _response;
            else
                error = _response.substring(0, endIndex);
        }

        return (error);

    } // end: getErrorLine


    /**
     * Accessor for the parsed message.
     *
     * @return              the parsed message
     */
    public String getMessage()
    {
        return (_response);

    } // end: getMessage


    /**
     * Parse the server message into a meaningful response.
     *
     * @param response      the server message to parse
     */
    public void parse(String response)
    {
        // Need to throw an error
        // Parse the string out
        int startIndex = response.indexOf("\n") + 1;
        int endIndex = response.lastIndexOf("\n") - 1;
        // If not message was returned
        if (endIndex <= startIndex)
            _response = "";
        else
            _response = response.substring(startIndex, endIndex).trim();

    } // end: parse


    /**
     * Indicate whether a command was completed successfully.
     *
     * @return              whether a command was completed successfully
     */
    public boolean successfulCompletion()
    {
        return (_response == "" );

    } // end: successfulCompletion


    /**
     * Overrides the default toString method to return the response.
     *
     * @return              the response message
     */
    public String toString()
    {
        return (_response);

    } // end: toString


} // end: TelnetResponse