/*
 * @(#)TelnetUUDecoder.java	1.0 2001/03/1
 */


package net.sourceforge.journeyftp.adapter.jta;


// Java imports
import java.io.*;
import sun.misc.*;


/**
 * Provides UUDecoder functionality for the telnet adapter.
 *
 * @author  Maru Newby
 * @version 1.0, 2001/03/31
 */
public class TelnetUUDecoder extends UUDecoder
{


    /**
     * Default no-arg constructor.
     */
    public TelnetUUDecoder()
    {
        super();
    }


    /**
     * Decode the text from the InputStream and write the decoded
     * octets to the OutputStream. This method runs until the stream
     * is exhausted.
     * @exception CEFormatException An error has occured while decoding
     * @exception CEStreamExhausted Input stream is unexpectedly out of data
     */
    public int _decodeBuffer(InputStream aStream, OutputStream bStream)
                             throws IOException
    {
        int	i;
        int	totalBytes = 0;
        int length;


        length = _decodeLinePrefix(aStream, bStream);
        for (i = 0; (i+bytesPerAtom()) < length; i += bytesPerAtom()) {
            decodeAtom(aStream, bStream, bytesPerAtom());
            totalBytes += bytesPerAtom();
            }
        if ((i + bytesPerAtom()) == length) {
            decodeAtom(aStream, bStream, bytesPerAtom());
            totalBytes += bytesPerAtom();
        } else {
            decodeAtom(aStream, bStream, length - i);
            totalBytes += (length - i);
            }
            _decodeLineSuffix(aStream, bStream);

        return (totalBytes);
    }


    /**
     * Decode a UU atom. Note that if l is less than 3 we don't write
     * the extra bits, however the encoder always encodes 4 character
     * groups even when they are not needed.
     */
    public void _decodeAtom(InputStream inStream,
                            OutputStream outStream, int l) throws IOException
    {
        decodeAtom(inStream, outStream, l);
    }


    /**
     * For uuencoded buffers, the data begins with a line of the form:
     * 		begin MODE FILENAME
     * This line always starts in column 1.
     */
    public void _decodeBufferPrefix(InputStream inStream,
                                    OutputStream outStream) throws IOException
    {
        decodeBufferPrefix(inStream, outStream);
    }




    /**
     * In uuencoded buffers, encoded lines start with a character that
     * represents the number of bytes encoded in this line. The last
     * line of input is always a line that starts with a single space
     * character, which would be a zero length line.
     */
    public int _decodeLinePrefix(InputStream inStream, OutputStream outStream) throws IOException {
    int	c;

    c = inStream.read();
    if (c == '`')
        c = ' ';
    if (c == ' ') {
        c = inStream.read();
        if (c == '\r')
            c = inStream.read();
        throw new CEStreamExhausted();
    } else if (c == -1) {
        throw new CEFormatException("UUDecoder: Short Buffer.");
    }

    int oldc;
    c = (c - ' ') & 0x3f;
    if (c > bytesPerLine()) {
        throw new CEFormatException("UUDecoder: Bad Line Length.");
    }
    return (c);
    }


    /**
     * Find the end of the line for the next operation.
     */
    public  void _decodeLineSuffix(InputStream inStream, OutputStream outStream) throws IOException {
    int c;
    while (true) {
        c = inStream.read();
        if (c == -1) {
        throw new CEStreamExhausted();
        }
        if (c == '\r') {
            c = inStream.read();
            if (c == -1) {
            throw new CEStreamExhausted();
            }
            if (c == '\n') {
            break;
            }
        }
        if (c == '\n') {
        break;
        }
    }
    }


    /**
     * UUencoded files have a buffer suffix which consists of the word
     * end. This line should immediately follow the line with a single
     * space in it.
     */
    public void _decodeBufferSuffix(InputStream inStream,
                                    OutputStream outStream) throws IOException
    {
        decodeBufferSuffix(inStream, outStream);
    }


} // end: TelnetUUDecoder
