/*
 * @(#)TelnetResponse.java	1.0 2001/03/21
 *
 * Copyright (C) 2001 by Maru Newby
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To contact the author, email ekko@users.sourceforge.net.
 */


package net.sourceforge.journeyftp.adapter.resources;


// Java imports
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.StringTokenizer;


/**
 * Provides string resources for adapters
 *
 * @author  Maru Newby
 * @version 1.0, 2001/03/21
 */
public class AdapterResourceBundle
{
    // Class variables
    public final static String ADAPTER_MSG_BUNDLE =
               "net.sourceforge.journeyftp.adapter.resources.AdapterMsgBundle";
    private static ResourceBundle _bundle = null;


    /**
     * Accessor for the primary resource bundle.
     *
     * @return the primary resource bundle
     */
    private static ResourceBundle getResourceBundle()
    {
        if (_bundle == null)
        {
            try
            {
                _bundle = ResourceBundle.getBundle(ADAPTER_MSG_BUNDLE);
            }
            catch (MissingResourceException mre)
            {
                // Message strings won't be available
                System.out.println("Unable to load resource bundle " +
                                   ADAPTER_MSG_BUNDLE);
            }
        }

        return (_bundle);

    } // end: getResourceBundle


    /**
     * Returns a value for a key from the default resource bundle.
     *
     * @param key       the key of the resource string
     * @return the resource string
     */
    public static String getResourceString(String key)
    {
        return (getResourceString(getResourceBundle(), key));

    } // end: getResourceString


    /*
     * Looks up a resource string and performs parameter substitution from the
     * provided array.  Out of order substitution is supported.  Use of
     * substitution keys ("{i}") can result in unexpected behaviour.
     *
     * Example usage:
     * Resource string is: "I have a {0}, a {2}, and a {1}."
     * Parameter array is: {"cat","dog","parrot"}
     * Resulting string is: "I have a cat, a parrot, and a dog"
     *
     * @param key       the key of the resource string
     * @param param     the array of substitution values.
     * @return the substituted resource string
     */
    public static String getResourceString(String key, String param[])
    {
        String str = getResourceString(key);
        StringBuffer buff = new StringBuffer(str);
        String substKey = "";
        int start = 0;

        for (int i = 0; i < param.length; ++i)
        {
            substKey = "{" + i + "}";
            start = str.indexOf(substKey);
            if (start != -1)
            {
                buff.replace(start, start + substKey.length(), param[i]);
                str = buff.toString();
            }
        }

        return (buff.toString());

    } // end: getResourceString


    /**
     * Returns a value for a key from the given resource bundle.
     *
     * @param bundle    the resource bundle to search
     * @param key       the key of the resource string
     * @return the resource string
     */
    private static String getResourceString(ResourceBundle bundle, String key)
    {
        String resource = null;
        try
        {
            if (bundle != null)
                resource = bundle.getString (key).trim();
            else
                resource = key;

        } catch (MissingResourceException ex)
        {
            resource = key;
        }

        return (resource);

    } // end: getResourceString


} // end: AdapterResourceBundle