/*
 * @(#)WFSFileSender.java	1.0 2001/03/12
 *
 * Copyright (C) 2001 by David Tarc, Maru Newby
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To contact the author, email ekko@users.sourceforge.net.
 */


package net.sourceforge.journeyftp.adapter.sambawfs;


// Java imports
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

// Fooware imports
import jcifs.smb.*;

// Adapter imports
import net.sourceforge.journeyftp.adapter.ConnectionFailedException;
import net.sourceforge.journeyftp.adapter.FileSender;
import net.sourceforge.journeyftp.adapter.TransferAdapterException;


/**
 * WFS implementation of asynchronous file sender.
 *
 * @author  David Tarc, Maru Newby
 * @version 1.0, 2001/03/15
 */
public class WFSFileSender extends FileSender
{
    // Class variables
    public final static int BUFF_SIZE = 8192;

    /**
     * Sends the data from the input stream to the server, where it
     * will be written to a file identified by fileName in the current working
     * directory of the server.  If the file already exists on the server, it
     * will be overwritten if permitted.
     *
     * If the offset is non-zero, the contents of that file will be written to
     * the server from that offset.  It is assumed that the input stream
     * has already been set to the desired offset.
     *
     * @exception TransferAdapterException  an error occured that terminated
     *                                      the file transfer
     */
    public void putFile() throws TransferAdapterException
    {
        byte[] buffer = new byte[BUFF_SIZE];
        int bytesRead;

        // Retrieve the parameters
        InputStream in = this.getInputStream();
        String fileName = this.getFileName();
        long offset = this.getOffset();

        // Retrieve the client object
        WFSFileTransferConnection connection =
                                (WFSFileTransferConnection)this.getConnection();
        SmbFile fileToPut = connection.getSmbFile( fileName );

        try
        {
            boolean append;
            if ( offset > 0L )
            {
                append = true;
            } else
            {
                append = false;
            }

            SmbFileOutputStream smbOut = new SmbFileOutputStream(fileToPut, append);

            while ((! this.transferAbortRequested()) &&
                  (bytesRead = in.read(buffer, 0, buffer.length)) != -1)
            {
                smbOut.write( buffer, 0, bytesRead );
                this.notifyDataTransferred(bytesRead);
            }

            smbOut.close();

            // If no exception is thrown before here, transfer was successful
        }
        catch (IOException ioe)
        {
            throw (new ConnectionFailedException(ioe.getMessage()));
        }

    } // end: putFile


} // end: WFSFileSender
