/*  *******************************************************************  */
/*  ======================== FTPBackend.java ==========================  */
/*  *******************************************************************  */

/* This class main class for implementation of backend of GUI */
/* It uses methods from classes JourneyObserver, ServerReply */
/* and QueuedFiles. Its main purposes are to send commands to */
/* the server, get server responses and give the user messages. */

/* It should override some methods defined in JourneyObserver class. */

package net.sourceforge.journeyftp.backend;

import java.util.*;
import java.net.*;
import java.io.*;
import javax.swing.*;

import net.sourceforge.journeyftp.gui.MessageManager;
import net.sourceforge.journeyftp.gui.TransferObserver;
import net.sourceforge.journeyftp.adapter.*;

// FTPBackend
//public class FTPBackend implements FileTransferObserver{
public class FTPBackend implements FileTransferObserver {

   /* class data members */
   private static final String defDirFile = new String ("defDir.dat");
   private static TransferAdapter backendAdapter = null;
   private ConnectionInfo currentConnection = null;
   private boolean putUnderway =false;
   private boolean getUnderway =false;
   private Vector toPutFiles = new Vector(0);
   private Vector toGetFiles = new Vector(0);
   private TransferObserver bigBrother;
   private MessageManager errorHandler;
   private ConInfoList hostList;
   private boolean paused = false;
   private String helpFileLoc;

    /* constructor */
    public FTPBackend()
    {


    }

    public FTPBackend(MessageManager middleMan)
    {


	helpFileLoc = locPwd();
        this.changeLocDir(getDefaultDir());
        errorHandler = middleMan;

        //in here we will have to read the stuff from the disk into the ten most recently accessed hosts
        hostList = new ConInfoList();
        hostList = hostList.fromDisk();

        if (hostList == null)
        {
        	hostList = new ConInfoList();
        	System.out.println("I ret rnul.");
        }

    }


    public String getHelpFileLoc()
    {
    	return helpFileLoc;
    }

   /* change the current directory of the client */
   public void changeLocDir (String dirName)
   {
      String olddir;
      olddir = System.getProperty("user.dir");

      File dirFile = new File(System.getProperty("user.dir"));

      if (dirName.equals(".."))
      {
         try
         {
         	System.setProperty("user.dir", (new File (dirFile.getParent())).getCanonicalPath());
         }
         catch (IOException e)
         {
                 System.setProperty("user.dir", (new File (dirFile.getParent())).getAbsolutePath());
         }
      }
      else
      {
        File new_dir = null;

        if (dirName.startsWith(File.separator))
        {
        	new_dir = new File (dirName);
        }
        else
        {
            new_dir = new File(olddir + File.separator + dirName);
	}

         if (!new_dir.exists())
         {
            errorHandler.errorMessage("Local" + new_dir.getAbsolutePath() +" change directory didn't work because directory didn't exist");
         }
         else if (!new_dir.isDirectory())
         {
            errorHandler.errorMessage("Cannot change into that directory because that file is not a directory!");
         }
    else
    {
        try
        {
        System.setProperty("user.dir",new_dir.getCanonicalPath());
        }
        catch (IOException e)
        {
        System.setProperty("user.dir",new_dir.getAbsolutePath());
        }
    }
      }

      System.out.println("Current directory is " + System.getProperty("user.dir"));


   } //changeLocDir


   /* change the current directory of the server */
   public void changeRemDir (String dirName)
   {
      boolean success = true;

      try
      {
         backendAdapter.setWorkingDirectory(dirName);
      }

	catch (TransferAdapterException e)
	{
 		errorHandler.errorMessage(e.getMessage());
 		success = false;
 	}
	

      if(success) {
       errorHandler.windowMessage("Remote CWD successful.");
      }
   } // changeRemDir


    /* delete files in the local directory */
    public boolean deleteLocFile(VirtualFile[] oldFile)
    {
        boolean worked = true;;
        for(int i = 0; i < oldFile.length; i++)
        {
                if(oldFile[i].isDirectory())
                {
                    changeLocDir(oldFile[i].getName());
                    deleteLocFile(listLocFiles());
                    changeLocDir("..");
            }

                String currentDir = System.getProperty("user.dir");
                File temp = new File(currentDir + File.separator + oldFile[i].getName());
            if(temp.delete())
            {
                errorHandler.windowMessage("Deleting local file succeeded.");
                worked = true;
            }
            else
            {
              errorHandler.errorMessage("Error: deleting local file failed.");
              worked = false;
             }
        }
        return worked;

    }

    // delete files in the remote directory */
    public boolean deleteRemFile (VirtualFile[] oldFile)
    {
        boolean success = true;

        for(int i = 0; i < oldFile.length; i++)
        {
                if(oldFile[i].isDirectory())
                {
                    changeRemDir(oldFile[i].getName());
                    deleteRemFile(listRemFiles());
                    changeRemDir("..");
		}

		try
		{
                    backendAdapter.remove(oldFile[i]);
		}
		catch (DirectoryNotEmptyException e)
		{
			success = false;
		}
		catch (TransferAdapterException e)
		{
			success = false;
		}

        	if(!success)
        	{
			errorHandler.errorMessage("Error: Deleting remote file " + oldFile[i].getName() + " failed.");
		}
        }
	if(success) {
		errorHandler.windowMessage("Deleting file successfully");	
	}
	return success;

    }


   // will initiate a connection on one of the three available protocols
   public boolean doConnect(ConnectionInfo info)
   {

      boolean success = true;
      TransferAdapterInfo[] aInfo = null;
      currentConnection = info;

      hostList.addOn(info);

      try
      {

         aInfo = TransferAdapterFactory.getInstance().getAdapterList();
      }
      catch (Exception e)
      {
          success = false;
      }

      if(success)
      {
        if (info.getProtocol().equals("Automatic"))
        {
		success = false;
		int i = aInfo.length;

	        while (!success && i  > 0)
        	{
			i--;
			success = true;
			try
			{
				backendAdapter = TransferAdapterFactory.getInstance().createTransferAdapter(aInfo[i]);
				backendAdapter.connect(info);
			}

			catch (TransferAdapterException e)
			{
 				success = false;
			}			
//			catch (Exception e)
//			{
//				errorHandler.errorMessage(e.getMessage());
//				success = false;
//			}
                }
        }

        else if (info.getProtocol().equals("FTP")) {
                try
                {
                    backendAdapter = TransferAdapterFactory.getInstance().createTransferAdapter(aInfo[2]);
                    backendAdapter.connect(info);
                }

		catch (TransferAdapterException e)
		{
 			success = false;
		}			
//                catch (Exception e)
//                {
//	            errorHandler.errorMessage("Error :" + e +" " + e.getMessage());
//                    success = false;
//                }
        }
        else if (info.getProtocol().equals("Telnet")) {
		try
		{
			backendAdapter = TransferAdapterFactory.getInstance().createTransferAdapter(aInfo[1]);
			backendAdapter.connect(info);
		}

		catch (TransferAdapterException e)
		{
 			success = false;		
		}		

        }
        else { // WFS
		try
		{
			backendAdapter = TransferAdapterFactory.getInstance().createTransferAdapter(aInfo[0]);
			backendAdapter.connect(info);
		}

		catch (TransferAdapterException e)
		{
                	success = false;

		}
	}
      }

      if(success)
      {
        errorHandler.windowMessage("Connection successful.");
	System.out.println("Connect as " + backendAdapter.getClass());
	backendAdapter.addObserver(this);
      }
      else
      {
        errorHandler.errorMessage("Error : Connection Unsuccessfully.");
     }

     return success;

   } // doConnect

   /* Disconnect to the server */
   public void doDisconnect()
   {

       backendAdapter.disconnect();
       errorHandler.windowMessage("Disconnect successfully");
   } //doDisconnect


    public ConnectionInfo getConnectionInfo ()
    {
        return currentConnection;
    }


    public String getDefaultDir ()
    {
        String defdir = new String("");
        File dataFile = new File (defDirFile);
        if (dataFile.exists())
	{
	        try
	        {
	        	BufferedReader readData= new BufferedReader(new FileReader( dataFile ));
	                try
		        {
	        		defdir = readData.readLine();
	        		readData.close();
		        }
		        catch (IOException e)
	        	{
				errorHandler.errorMessage("Warning : Connection Unsuccessfully.");
		        }       		
	        }
	        catch (FileNotFoundException e)
	        {
			errorHandler.errorMessage("Warning : Connection Unsuccessfully.");  	        	
	        }
	
        }
        else
        {
        	defdir=  locPwd();
        }
        return defdir;
    }

	public void getFile(VirtualFile[] toGet, TransferObserver theDyer)
	{
	    	long totalSize;
	    	
	    	bigBrother = theDyer;		
		totalSize= getFileList(toGet);    	
	    	bigBrother.batchInitiated(totalSize);
 	    	VirtualFile currentFile = (VirtualFile)toGetFiles.firstElement();    	
	    	bigBrother.transferInitiated(currentFile.getResidentDir(), currentFile.getDestinationDir(), currentFile.getSize(), currentFile.getName());
	    	getUnderway = true;
	    	commitGetFile(currentFile);
	}

    public boolean commitGetFile(VirtualFile currentFile)
    {
	boolean worked;

 	FileOutputStream fos = null;
        File file = new File(currentFile.getDestinationDir() + File.separator + currentFile.getName());
        changeRemDir(currentFile.getResidentDir());
                if(file.exists())
                {
                    String toDo;
                    String[] choices = {"Overwrite", "Resume", "Abort"};
                    toDo = errorHandler.optionMessage( currentFile + ": Already exists.", choices );

                    if (toDo.equals("Overwrite"))
                    {
                    	
               		VirtualFile[] toDel = new VirtualFile[1];
                    	toDel[0] = currentFile;
                    	worked = deleteLocFile(toDel);
                    	if (!worked)
                    	{
				errorHandler.errorMessage("The overwrite attempt failed.");
                    	}
                    	else
                    	{
	                    	worked = commitGetFile(currentFile);	                    	
                    	}
                    	
                    }
                    else if (toDo.equals("Resume"))
                    {
	   	     try
       	             {
       	                 fos = new FileOutputStream( file );
       	                 backendAdapter.getFile( currentFile, fos, file.length() );
       	             }
       	             catch ( FileNotFoundException fnfe )
       	             {
       	                 errorHandler.errorMessage( currentFile.getName() + ": File not readable." );
       	                 return false;
       	             }
       	
       	             catch (TransferAdapterException e) // if this is a file exists exception we will need to prompt the user to overwrite/rename/resume the file
       	             {
       	                 errorHandler.errorMessage( e.getMessage() );
       	                 return false;
		     }
      	
       	             catch (IOException e)
       	             {
       	                 errorHandler.errorMessage( "The following IO exception occurred: " + e.getMessage() );
       	                 return false;
       	             }
       	           }
                   else if (toDo.equals("Abort"))
                   {
                   	return false;
                   }
       	        }                     	
                else
                {
                    try
                    {
                        fos = new FileOutputStream( file );
                        backendAdapter.getFile( currentFile, fos, 0 );
                    }
                    catch (TransferAdapterException e) // if this is a file exists exception we will need to prompt the user to overwrite/rename/resume the file
                    {
                        errorHandler.errorMessage( "The following transferadapterexception happened: " +e.getMessage() );
                        return false;
                    }

                     catch (IOException e)
                     {
                        errorHandler.errorMessage( "Fatal Transfer error, IO exception occurred: " + e.getMessage() );
                        return false;
                     }
	        }	
	        return true;

    	
    	//theDoer.getFile((VirtualFile)toGetFiles.firstElement(), backendAdapter, this.getConnectionInfo());
    }

    private long getFileList(VirtualFile[] toGet)
    {
        long totalSize =0;
        for (int i =0; i< toGet.length; i++)
        {
            String fileName = toGet[i].getName();
            //System.out.println("Remote file name is " + fileName);
            if (toGet[i].isDirectory()) //it is a directory, so don't do the usual get
            {
                //System.out.println("Remote " +  toGet[i].getName() + " is directory!");
                if (makeLocDir(toGet[i].getName()) || (new File (System.getProperty("user.dir") + File.separator + toGet[i].getName()).isDirectory() ))
                {
                    changeLocDir(toGet[i].getName());
                    changeRemDir(toGet[i].getName());
                    totalSize += getFileList(listRemFiles());
                    changeLocDir("..");
                    changeRemDir("..");
                }
            }
            else
            {
            	toGet[i].setDestinationDir(locPwd());
            	totalSize += toGet[i].getSize();
            	toGetFiles.addElement(toGet[i]);
            }
        }
        return totalSize;
    }

    public ConnectionInfo[] getHostList()
    {
	return hostList.display();
    }

    public String[] getHostListStrings()
    {
	return hostList.displayNames();
    }


    public boolean isTransferring()
    {
    	boolean busy;
    	busy = putUnderway || getUnderway;
    	return busy;
    }

    public boolean isConnected()
    {
	
	if (backendAdapter == null)	
	{
		if (currentConnection != null)
		{
			currentConnection.setConnected(false);
		}
		return false;
	}
	else
	{	
		if (currentConnection != null)
		{
			currentConnection.setConnected(backendAdapter.isConnected());
		}
		return backendAdapter.isConnected();
	}		
    }


   /* list files in the current directory of the client */
   public VirtualFile[] listLocFiles ()
   {
    File file = null;
      String currentDir = System.getProperty("user.dir");

      File dirFile = new File(currentDir);
      String[] nameList = dirFile.list();

      VirtualFile[] virtFiles = new VirtualFile[nameList.length];
      for( int i = 0; i < nameList.length; i++)
      {
           file = new File(currentDir + File.separator + nameList[i]);
           virtFiles[i] = new VirtualFile();
           virtFiles[i].setName(nameList[i]);
           virtFiles[i].setSize(file.length());
           virtFiles[i].setDateModified(new Date(file.lastModified()));
           virtFiles[i].setIsDirectory(file.isDirectory());
           virtFiles[i].setResidentDir(locPwd());
      }

      //errorHandler.windowMessage("Local list files successful.");

      return virtFiles;

   } // listLocFiles


   /* list files in the current directory of the server. */
   public VirtualFile[] listRemFiles ()
   {
      boolean success = true;
      VirtualFile[] files = new VirtualFile[0];
      try
      {
       	String remoteDir = remPwd();
         files = backendAdapter.list();
        for (int i=0;i<files.length;i++)
	{
 		files[i].setResidentDir(remoteDir);
	}			
      }
     catch (TransferAdapterException e)
     {
     	success = false;
	errorHandler.errorMessage("Error :" + e + " " + e.getMessage());
 	} 	
     return files;
   } // listRemFiles



    /*
    * This method should be called right after GUI is invoked.
    * the directory information will be displayed on the box below local side.
    */
   public String locPwd ()
   {
    return System.getProperty("user.dir");

   }



   /* make a directory in the client */
   public boolean makeLocDir (String dirName)
   {
    boolean success = true;
    String currentDir = System.getProperty("user.dir");
    File loc_dir = new File(currentDir + File.separator + dirName);

    if (loc_dir.mkdir())
    {
        errorHandler.windowMessage("Make local directory successful.");
    }
    else
    {
        success = false;
        errorHandler.errorMessage("Error: Make local directory failed.");
    }

    return success;

   } //makeLocDir


   /* make a directory in the server */
   public boolean makeRemDir (String dirName)
   {
      boolean success = true;

      try
      {
         backendAdapter.createDirectory (dirName);
      }
      catch (Exception e )
      {
         success = false;
         errorHandler.windowMessage("Error :" + e +" " + e.getMessage());
         errorHandler.errorMessage("Error : make new remote directory failed.");
      }
      if(success)
      {
         errorHandler.windowMessage("Make remote directory successful");
      }

      return (success);

   } //makeRemDir

	public void putFile(VirtualFile[] toPut, TransferObserver theDyer)
	{
              	long totalSize;
    		bigBrother = theDyer;
		totalSize = putFileList(toPut);
		VirtualFile currentFile = (VirtualFile)toPutFiles.firstElement();
    	        bigBrother.batchInitiated(totalSize);
    	        bigBrother.transferInitiated(currentFile.getResidentDir(), currentFile.getDestinationDir(), currentFile.getSize(), currentFile.getName());		
    		putUnderway = true;    			
		commitPutFile(currentFile);    		
	}
	
	
	
	public boolean commitPutFile(VirtualFile currentFile)
	{
		boolean worked = true;	
		FileInputStream fis = null;

	        File file = new File (currentFile.getResidentDir() + File.separator + currentFile.getName());
	        changeLocDir(currentFile.getDestinationDir());
	        if ( ! file.canRead() )
		{
	        	errorHandler.errorMessage( currentFile.getName() + ": Unable to read file." );
			return false;
		}				
		
		try
		{
        		fis = new FileInputStream( file );
		        backendAdapter.putFile( currentFile + File.separator  +currentFile.getName(), fis, 0 );
		        worked = true;
		}

		catch ( FileNotFoundException fnfe )
		{
			errorHandler.errorMessage( currentFile.getName() + ": File not found." );
			worked = false;
		}
   	

        catch (TransferAdapterException e)
        {

		errorHandler.windowMessage( " Error: " + e.getMessage());
		 worked = false;;
        }
        return worked;	
    }

   public long putFileList(VirtualFile[] toPut)
   {
    long totalSize = 0;
    for (int i =0; i< toPut.length; i++)
    {
        String filename = toPut[i].getName();

            if (toPut[i].isDirectory()) //it is a directory, so don't do the usual get
            {
                makeRemDir(toPut[i].getName());

                changeLocDir(toPut[i].getName());
                changeRemDir(toPut[i].getName());

                totalSize += putFileList(listLocFiles()); //append the directory of files to the list

                changeLocDir("..");
                changeRemDir("..");

            }
            else
            {
		totalSize += toPut[i].getSize();
		toPut[i].setDestinationDir(remPwd());
		toPutFiles.addElement(toPut[i]);

	    }
     }
     return totalSize;
   }

   /* exit the system */
   public void quit()
   {
    	hostList.toDisk();
    //write ConnectionInfo list to disk
   } //quit



      /*
    * This method should be called right after GUI is invoked.
    * the directory information will be displayed on the box below server side.
    */
   public String remPwd ()
   {
    String remWorkingDir = new String("");
    try
    {
        remWorkingDir = backendAdapter.getWorkingDirectory();
    }
    catch (Exception e)
    {
        errorHandler.errorMessage("Error: Connection is dead.");
        return ("");
    }
    return remWorkingDir;
   }




    /* rename files in the local directory */
    public void renameLocFile (VirtualFile oldFile, String newName)
    {
            String currentDir = System.getProperty("user.dir");
            File temp = new File(currentDir+ File.separator + oldFile.getName());
            File newFile = new File(currentDir+ File.separator + newName);

            // temp.renameTo(newFile);
        if (temp.renameTo(newFile)) {
            errorHandler.windowMessage("Renaming local file succeeded.");
          }
            else
            {
            errorHandler.errorMessage("Error: Renaming local file failed.");
            }
    }



    // rename files in the remote directory */
    public void renameRemFile (VirtualFile oldFile, String newName)
    {
        boolean success = true;
        try {
            backendAdapter.rename(oldFile.getName(), newName);
        }

        catch (Exception rfex) {
            errorHandler.windowMessage("renameRem: "+rfex.getMessage());
            success = false;
        }

        if (success) {
            errorHandler.windowMessage("Renaming remote file succeeded.");
        }

        else
        {
            errorHandler.errorMessage("Error: Renaming remote file failed.");
        }
    }


    public void setConnected(boolean bConnected)
    {
    	if (getConnectionInfo() != null)
    	{
	    	getConnectionInfo().setConnected(bConnected);
    	}
    }

    public void setDefaultDir (String newDefDir) throws IOException
    {
        File dataFile = new File (defDirFile);
        File newdir = new File (newDefDir);

        if (!newdir.isDirectory())
        {
            errorHandler.errorMessage("Warning: You have chosen an invalid directory for the default.");
        }
        else
        {
            PrintWriter out = new PrintWriter (new FileOutputStream (dataFile));
            dataFile.createNewFile();
            out.println(newDefDir);
            out.close();
            errorHandler.windowMessage("Set default directory successfully.");
        }
       }

   /* public void outputError(String errorstring)
    {
        System.out.println(errorstring);
    }*/

    /**
     * This method is called every time the <code>FileTransfer</code> object
     * received a block of bytes, allowing observers to monitor the status of
     * a file transfer.
     *
     * @param bytes         the number of bytes received
     */


    public void dataTransferred(long bytes)
    {
	/*try
	{
		this.wait(500);
	}
	catch (InterruptedException e)
	{
			
	} */
	//System.out.println("the adapter guys aren't fuckers");
	bigBrother.dataTransferred(bytes);    	
    }


    public void dataReceived(int bytes)
    {
      dataTransferred((long) bytes);
    } // end: dataReceived


    /**
     * This method is called every time the <code>FileTransfer</code> object
     * sends a block of bytes, allowing observers to monitor the status of
     * a file transfer.
     *
     * @param bytes         the number of bytes transferred
     */
    public void dataSent(int bytes)
    {
	dataTransferred((long) bytes);
    } // end: dataSent


    /**
     * Indicates that a file transfer has been completed successfully.
     */

    public void transferSuccessful()
    {	
	if (putUnderway)
	{
		bigBrother.transferFinished("Transfer of " + ((VirtualFile)toPutFiles.firstElement()).getName() + " completed successfully.");
	}
	else if (getUnderway)
	{
		bigBrother.transferFinished("Transfer of " + ((VirtualFile)toGetFiles.firstElement()).getName() + " completed successfully.");
	}
	else
	{
		bigBrother.transferFinished("You weren't tranferring anything !!!?!?!");	    		
	}
	doNextTransfer();
		
    } // end: transferSuccessful

    /**
     * Indicates that a file transfer has terminated.  The exception can be
     * used to determine why the transfer terminated.
     *
     * @param ex        the exception indicating the reason for the termination
     */
	
     public void doTheStop ()
     {
     	backendAdapter.abortTransfer();
     	paused = true;
     }

     public void doTheResume ()
     {
     	paused = false;
     	if (putUnderway)
     	{     	
     		commitPutFile((VirtualFile)toPutFiles.firstElement());
     	}
     	else if (getUnderway)
     	{
     		commitGetFile((VirtualFile)toGetFiles.firstElement());
     	}     	     	
     }
	
	public void transferTerminated(TransferAdapterException tae)
	{
		
		if (paused)
		{
			return;
		}
		//if TransferAdapterException
		
		System.out.println(tae.getClass().toString());
		bigBrother.transferFinished("Error : " + tae +  " " + tae.getMessage());
                doNextTransfer();


                        /*catch (FileAlreadyExistsException e)
            {
                    ask the user if he wants a rename/resume/overwrite, and if it's a rename, rename to what

                    if (choice.equals ("overwrite"))
                    {
                        VirtualFile[] delone = new VirtualFile[1];
                        delone[1] = toPut[i];
                        deleteRemFile(delone);
                        putFile(toPut[i]);
                    }
                    else if (choice.equals ("rename"))
                    {
                        renameRemFile(toPut[i], newfilename);
                        putFile(toPut[i]);
                    }

                    else if (choice.equals ( "resume" ))
                    {
                        fis = new FileInputStream ( file )
                        backend.putFile(toPut[i].getName(), fis, toPut[i].getLength());
                    }

            }    */


		
	} // end: transferTerminated

    private void doNextTransfer()
    {
       	VirtualFile currentFile;    	
	if (putUnderway)
	{		
		if (!toPutFiles.isEmpty())
		{
			toPutFiles.removeElementAt(0);	
		}
		if (toPutFiles.isEmpty())
		{
			putUnderway = false;
			bigBrother.batchFinished();
			System.out.println("I'm all done.");
		}
	}
	
        if (getUnderway)
        {
        	if (!toGetFiles.isEmpty())
        	{
	        	toGetFiles.removeElementAt(0);
        	}
        	if (toGetFiles.isEmpty())
        	{
			getUnderway = false;        	
			bigBrother.batchFinished();
			System.out.println("I'm all done.");
        	}
   	}
		
	if (putUnderway)
	{
		currentFile = (VirtualFile)toPutFiles.firstElement();
		bigBrother.transferInitiated(currentFile.getResidentDir(), currentFile.getDestinationDir(), currentFile.getSize(), currentFile.getName());
		commitPutFile(currentFile);	
		
	}
	else if (getUnderway)
	{
		currentFile = (VirtualFile)toGetFiles.firstElement();
		bigBrother.transferInitiated(currentFile.getResidentDir(), currentFile.getDestinationDir(), currentFile.getSize(), currentFile.getName());	
		commitGetFile(currentFile);	

	}


   }
	
} //FTPBackend.java

