/* ********************************************************* */
/* ================== QueueInfo.java ===================== */
/* ********************************************************* */

/*  This class deals with queued file operations during download */

package net.sourceforge.journeyftp.backend;

import java.io.*;
import java.util.*;
import java.text.SimpleDateFormat;
import java.text.DateFormat;

import net.sourceforge.journeyftp.adapter.*;
import net.sourceforge.journeyftp.adapter.fooftp.*;

//QueuedFiles
public class QueueInfo {

   private VirtualFile q_vfile = null; //stored file structure
   private ConnectionInfo q_info = null; //connection information
   private String locPath = null; //local directory path
   private String remPath = null; //remote directory path
   private String request = null; //"get" or "put"

  /* empty constructor */
  public QueueInfo ()
  {
  }


  /* constructor with Virtualfile and ConnectionInfo objects */
  public QueueInfo (VirtualFile vfile, ConnectionInfo info, String from_path, String to_path, String getorput)
  {
     q_vfile = vfile;
     q_info = info;
     locPath = from_path;
     remPath = to_path;
     request = getorput;
	q_vfile.setDestinationDir(to_path);

  }

  public ConnectionInfo getConnectionInfo()
  {
  	return q_info;
  }

  /* get filename */
  public VirtualFile getVirtualFile()
  {
 	return q_vfile;

  }


  public String getName()
  {
	return  q_vfile.getName();
  } //getName


  /* get file length */
  public long getLength()
  {
	 return q_vfile.getSize();
  } //getLength


  /* get last modified date */
  public Date getLastModifiedDate()
  {
	  return q_vfile.getDateModified();
  } //getLastModifiedDate


  /* get boolean if it is a directory */
  public boolean isDir()
  {
       return q_vfile.isDirectory();
  }  //isDirectory


  /* get local path */
  public String getLocPath()
  {
          return locPath;
  } //getLocPath


  /* get remote path */
  public String getRemPath()
  {
          return remPath;
  } //getRemPath


  /* get request: get or put */
  public String getRequest()
  {
	  return request;
  } //getRequest


  /* get host name */
  public String getHostname()
  {
	  return q_info.getHost();
  } // getHostname


  /* get user name */
  public String getUser()
  {
	  return q_info.getUsername();
  } //getUser


  /* get password */
  public String getPassword()
  {
	  return q_info.getPassword();
  } // getPassword


  /* get protocol */
  public String getProtocol()
  {
	  return q_info.getProtocol();
  } // getProtocol


} //QueueInfo
